@@ -1,4 +1,4 @@
-/* 
+/*
    Socket handling routines
    Copyright (C) 1998-2009, Joe Orton <joe@manyfish.co.uk>
    Copyright (C) 2004 Aleix Conchillo Flaque <aleix@member.fsf.org>
@@ -7,7 +7,7 @@
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -27,6 +27,9 @@
 #include "config.h"
 
 #include <sys/types.h>
+#ifdef HAVE_SYS_UIO_H
+#include <sys/uio.h> /* writev(2) */
+#endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -35,6 +38,11 @@
 #include <sys/socket.h>
 #endif
 
+#ifdef __linux__
+#include <sys/ioctl.h>
+#include <linux/sockios.h>
+#endif
+
 #ifdef NE_USE_POLL
 #include <sys/poll.h>
 #elif defined(HAVE_SYS_SELECT_H)
@@ -71,7 +79,7 @@
 #endif
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
-#endif 
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -167,12 +175,14 @@ typedef struct in_addr ne_inet_addr;
 #endif
 #define NE_ISRESET(e) ((e) == ECONNRESET || (e) == ECONNABORTED || (e) == ENOTCONN)
 #define NE_ISCLOSED(e) ((e) == EPIPE)
-#define NE_ISINTR(e) ((e) == EINTR)
+#define NE_ISINTR(e) (((e) == EINTR) || ((e) == EAGAIN))
 #define NE_ISINPROGRESS(e) ((e) == EINPROGRESS)
 #endif
 
 /* Socket read timeout */
-#define SOCKET_READ_TIMEOUT 120
+#define SOCKET_READ_TIMEOUT 3600
+/* Socket step timeout */
+#define SOCKET_STEP_TIMEOUT 10
 
 /* Critical I/O functions on a socket: useful abstraction for easily
  * handling SSL I/O alongside raw socket I/O. */
@@ -188,7 +198,7 @@ struct iofns {
     int (*readable)(ne_socket *s, int n);
     /* Write up to 'count' blocks described by 'vector' to socket.
      * Return number of bytes written on success, or <0 on error. */
-    ssize_t (*swritev)(ne_socket *s, const struct ne_iovec *vector, 
+    ssize_t (*swritev)(ne_socket *s, const struct ne_iovec *vector,
                        int count);
 };
 
@@ -239,7 +249,7 @@ static void print_error(int errnum, char *buffer, size_t buflen)
 {
     if (FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM
                        | FORMAT_MESSAGE_IGNORE_INSERTS,
-                       NULL, (DWORD) errnum, 0, 
+                       NULL, (DWORD) errnum, 0,
                        buffer, buflen, NULL) == 0)
         ne_snprintf(buffer, buflen, "Socket error %d", errnum);
 }
@@ -287,7 +297,7 @@ static int ipv6_disabled = 0;
 static void init_ipv6(void)
 {
     int fd = socket(AF_INET6, SOCK_STREAM, 0);
-    
+
     if (fd < 0)
         ipv6_disabled = 1;
     else
@@ -315,14 +325,14 @@ int ne_sock_init(void)
     if (init_state > 0) {
         init_state++;
 	return 0;
-    } 
+    }
     else if (init_state < 0) {
 	return -1;
     }
 
-#ifdef WIN32    
+#ifdef WIN32
     wVersionRequested = MAKEWORD(2, 2);
-    
+
     err = WSAStartup(wVersionRequested, &wsaData);
     if (err != 0) {
 	return init_state = -1;
@@ -339,7 +349,11 @@ int ne_sock_init(void)
 #endif
 
 #if defined(HAVE_SIGNAL)
-    (void) signal(SIGPIPE, SIG_IGN);
+    //(void) signal(SIGPIPE, SIG_IGN);
+    struct sigaction sa;
+    memset( &sa, 0, sizeof(sa) );
+    sa.sa_handler = SIG_IGN;
+    sigaction( SIGPIPE, &sa, NULL);
 #endif
 
 #ifdef USE_CHECK_IPV6
@@ -365,7 +379,7 @@ void ne_sock_exit(void)
 #ifdef NE_HAVE_SSL
         ne__ssl_exit();
 #endif
-        
+
 #ifdef HAVE_SSPI
         ne_sspi_deinit();
 #endif
@@ -380,15 +394,18 @@ static int raw_poll(int fdno, int rdwr, int secs)
     int ret;
 #ifdef NE_USE_POLL
     struct pollfd fds;
-    int timeout = secs > 0 ? secs * 1000 : -1;
+    int time_left = (secs > 0)?(secs):(-1);
 
     fds.fd = fdno;
     fds.events = rdwr == 0 ? POLLIN : POLLOUT;
     fds.revents = 0;
 
     do {
-        ret = poll(&fds, 1, timeout);
-    } while (ret < 0 && NE_ISINTR(ne_errno));
+        ret = poll(&fds, 1, 1000);
+        if( ret < 0 && !NE_ISINTR(ne_errno)){
+            break; // error
+        }
+    } while (ret <= 0 && --time_left);
 #else
     fd_set rdfds, wrfds;
     struct timeval timeout, *tvp = (secs >= 0 ? &timeout : NULL);
@@ -424,7 +441,7 @@ int ne_sock_block(ne_socket *sock, int n)
     return sock->ops->readable(sock, n);
 }
 
-/* Cast address object AD to type 'sockaddr_TY' */ 
+/* Cast address object AD to type 'sockaddr_TY' */
 #define SACAST(ty, ad) ((struct sockaddr_##ty *)(ad))
 
 ssize_t ne_sock_read(ne_socket *sock, char *buffer, size_t buflen)
@@ -432,7 +449,7 @@ ssize_t ne_sock_read(ne_socket *sock, char *buffer, size_t buflen)
     ssize_t bytes;
 
 #if 0
-    NE_DEBUG(NE_DBG_SOCKET, "buf: at %d, %d avail [%s]", 
+    NE_DEBUG(NE_DBG_SOCKET, "buf: at %d, %d avail [%s]",
 	     sock->bufpos - sock->buffer, sock->bufavail, sock->bufpos);
 #endif
 
@@ -458,14 +475,14 @@ ssize_t ne_sock_read(ne_socket *sock, char *buffer, size_t buflen)
 	memcpy(buffer, sock->buffer, buflen);
 	sock->bufpos = sock->buffer + buflen;
 	sock->bufavail = bytes - buflen;
-	return buflen; 
+	return buflen;
     }
 }
 
 ssize_t ne_sock_peek(ne_socket *sock, char *buffer, size_t buflen)
 {
     ssize_t bytes;
-    
+
     if (sock->bufavail) {
 	/* just return buffered data. */
 	bytes = sock->bufavail;
@@ -499,10 +516,50 @@ static int readable_raw(ne_socket *sock, int secs)
     return (ret == 0) ? NE_SOCK_TIMEOUT : 0;
 }
 
+#ifdef __linux__
+static int wait_pending_writes(ne_socket *sock, int stepwait)
+{
+    /* If there's any data in the TCP queue,
+       wait until it's been drained */
+
+    int prevpending = INT_MAX, pending = INT_MAX;
+    ssize_t ret;
+    do {
+        /* Ready to read from socket? */
+        ret = readable_raw(sock, stepwait);
+        if(ret == NE_SOCK_ERROR) return ret;
+
+        /* There was a timeout.. */
+        if(ret == NE_SOCK_TIMEOUT) {
+            /* Is there any data pending for writing? */
+            int ioret = ioctl(sock->fd, SIOCOUTQ, &pending);
+            if(ioret == -1) {
+                set_strerror(sock, ne_errno);
+                return NE_SOCK_ERROR;
+            }
+            if(pending > 0 && pending < prevpending) {
+                if(prevpending == INT_MAX) prevpending = -1;
+                NE_DEBUG(NE_DBG_SOCKET, "Pending bytes on TCP send buffer, waiting. (%d bytes)", pending);
+                prevpending = pending;
+            }
+            else {
+                /* Timeout, no change in buffer size since last time.
+                   Connection is probably hung */
+                return NE_SOCK_TIMEOUT;
+            }
+        }
+    } while(ret == NE_SOCK_TIMEOUT);
+    return 0;
+}
+#endif
+
 static ssize_t read_raw(ne_socket *sock, char *buffer, size_t len)
 {
     ssize_t ret;
-    
+#ifdef __linux__
+    ret = wait_pending_writes(sock, sock->rdtimeout);
+    if (ret) return ret;
+#endif
     ret = readable_raw(sock, sock->rdtimeout);
     if (ret) return ret;
 
@@ -525,10 +582,10 @@ static ssize_t read_raw(ne_socket *sock, char *buffer, size_t len)
 #define MAP_ERR(e) (NE_ISCLOSED(e) ? NE_SOCK_CLOSED : \
                     (NE_ISRESET(e) ? NE_SOCK_RESET : NE_SOCK_ERROR))
 
-static ssize_t write_raw(ne_socket *sock, const char *data, size_t length) 
+static ssize_t write_raw(ne_socket *sock, const char *data, size_t length)
 {
     ssize_t ret;
-    
+
 #ifdef __QNX__
     /* Test failures seen on QNX over loopback, if passing large
      * buffer lengths to send().  */
@@ -553,7 +610,7 @@ static ssize_t write_raw(ne_socket *sock, const char *data, size_t length)
     return ret;
 }
 
-static ssize_t writev_raw(ne_socket *sock, const struct ne_iovec *vector, int count) 
+static ssize_t writev_raw(ne_socket *sock, const struct ne_iovec *vector, int count)
 {
     ssize_t ret;
 #ifdef WIN32
@@ -565,11 +622,11 @@ static ssize_t writev_raw(ne_socket *sock, const struct ne_iovec *vector, int co
         wasvector[i].buf = vector[i].base;
         wasvector[i].len = vector[i].len;
     }
-        
+
     ret = WSASend(sock->fd, wasvector, count, &total, 0, NULL, NULL);
     if (ret == 0)
         ret = total;
-    
+
     ne_free(wasvector);
 #else
     const struct iovec *vec = (const struct iovec *) vector;
@@ -584,12 +641,12 @@ static ssize_t writev_raw(ne_socket *sock, const struct ne_iovec *vector, int co
 	set_strerror(sock, errnum);
 	return MAP_ERR(errnum);
     }
-    
+
     return ret;
 }
 
 #ifdef NE_HAVE_SSL
-static ssize_t writev_dummy(ne_socket *sock, const struct ne_iovec *vector, int count) 
+static ssize_t writev_dummy(ne_socket *sock, const struct ne_iovec *vector, int count)
 {
     return sock->ops->swrite(sock, vector[0].base, vector[0].len);
 }
@@ -616,7 +673,7 @@ static int error_ossl(ne_socket *sock, int sret)
 	set_error(sock, _("Connection closed"));
         return NE_SOCK_CLOSED;
     }
-    
+
     /* for all other errors, look at the OpenSSL error stack */
     err = ERR_get_error();
     if (err == 0) {
@@ -634,13 +691,13 @@ static int error_ossl(ne_socket *sock, int sret)
     }
 
     if (ERR_reason_error_string(err)) {
-        ne_snprintf(sock->error, sizeof sock->error, 
+        ne_snprintf(sock->error, sizeof sock->error,
                     _("SSL error: %s"), ERR_reason_error_string(err));
     } else {
-	ne_snprintf(sock->error, sizeof sock->error, 
+	ne_snprintf(sock->error, sizeof sock->error,
                     _("SSL error code %d/%d/%lu"), sret, errnum, err);
     }
-    
+
     /* make sure the error stack is now empty. */
     ERR_clear_error();
     return NE_SOCK_ERROR;
@@ -656,7 +713,7 @@ static ssize_t read_ossl(ne_socket *sock, char *buffer, size_t len)
 
     ret = readable_ossl(sock, sock->rdtimeout);
     if (ret) return ret;
-    
+
     ret = SSL_read(sock->ssl, buffer, CAST2INT(len));
     if (ret <= 0)
 	ret = error_ossl(sock, ret);
@@ -721,7 +778,7 @@ static ssize_t error_gnutls(ne_socket *sock, ssize_t sret)
 	break;
     case GNUTLS_E_FATAL_ALERT_RECEIVED:
         ret = NE_SOCK_ERROR;
-        ne_snprintf(sock->error, sizeof sock->error, 
+        ne_snprintf(sock->error, sizeof sock->error,
                     _("SSL alert received: %s"),
                     gnutls_alert_get_name(gnutls_alert_get(sock->ssl)));
         break;
@@ -758,12 +815,12 @@ static ssize_t read_gnutls(ne_socket *sock, char *buffer, size_t len)
 
     ret = readable_gnutls(sock, sock->rdtimeout);
     if (ret) return ret;
-    
+
     do {
         do {
             ret = gnutls_record_recv(sock->ssl, buffer, len);
         } while (RETRY_GNUTLS(sock, ret));
-        
+
     } while (ret == GNUTLS_E_REHANDSHAKE && reneg--
              && (ret = gnutls_handshake(sock->ssl)) == GNUTLS_E_SUCCESS);
 
@@ -823,7 +880,7 @@ int ne_sock_fullwritev(ne_socket *sock, const struct ne_iovec *vector, int count
                 count--;
                 vector++;
             }
-            
+
             if (ret && count) {
                 /* Partial buffer sent; send the rest. */
                 ret = ne_sock_fullwrite(sock, (char *)vector[0].base + ret,
@@ -852,9 +909,9 @@ ssize_t ne_sock_readline(ne_socket *sock, char *buf, size_t buflen)
 	if (sock->bufavail)
 	    memmove(sock->buffer, sock->bufpos, sock->bufavail);
 	sock->bufpos = sock->buffer;
-	
+
 	/* Loop filling the buffer whilst no newline is found in the data
-	 * buffered so far, and there is still buffer space available */ 
+	 * buffered so far, and there is still buffer space available */
 	do {
 	    /* Read more data onto end of buffer. */
 	    ssize_t ret = sock->ops->sread(sock, sock->buffer + sock->bufavail,
@@ -883,7 +940,7 @@ ssize_t ne_sock_readline(ne_socket *sock, char *buf, size_t buflen)
     return len;
 }
 
-ssize_t ne_sock_fullread(ne_socket *sock, char *buffer, size_t buflen) 
+ssize_t ne_sock_fullread(ne_socket *sock, char *buffer, size_t buflen)
 {
     ssize_t len;
 
@@ -927,7 +984,7 @@ ne_sock_addr *ne_addr_resolve(const char *hostname, int flags)
         hints.ai_family = AF_INET6;
 	addr->errnum = getaddrinfo(hn, NULL, &hints, &addr->result);
 	ne_free(hn);
-    } else 
+    } else
 #endif /* AF_INET6 */
     {
 #ifdef USE_GAI_ADDRCONFIG /* added in the RFC3493 API */
@@ -942,7 +999,7 @@ ne_sock_addr *ne_addr_resolve(const char *hostname, int flags)
 #else /* Use gethostbyname() */
     in_addr_t laddr;
     struct hostent *hp;
-    
+
     laddr = inet_addr(hostname);
     if (laddr == INADDR_NONE) {
 	hp = gethostbyname(hostname);
@@ -1084,7 +1141,7 @@ ne_inet_addr *ne_iaddr_parse(const char *addr, ne_iaddr_type type)
     if (inet_pton(af, addr, dst) != 1) {
         return NULL;
     }
-    
+
     return ne_iaddr_make(type, (unsigned char *)dst);
 #elif defined(USE_GETADDRINFO) && !defined(HAVE_INET_PTON)
     /* For Windows, which lacks inet_pton(). */
@@ -1094,11 +1151,11 @@ ne_inet_addr *ne_iaddr_parse(const char *addr, ne_iaddr_type type)
     hints.ai_socktype = SOCK_STREAM;
     hints.ai_flags = AI_NUMERICHOST;
     hints.ai_family = type == ne_iaddr_ipv6 ? AF_INET6 : AF_INET;
-    
+
     if (getaddrinfo(addr, NULL, &hints, &ai)) {
         return NULL;
     }
-    
+
     /* Copy the returned addrinfo, since it needs to be ne_free()-able
      * later; must only call freeaddrinfo() on ai. */
     rv = ne_calloc(sizeof *rv);
@@ -1108,11 +1165,11 @@ ne_inet_addr *ne_iaddr_parse(const char *addr, ne_iaddr_type type)
     rv->ai_addr = ne_calloc(ai->ai_addrlen);
     memcpy(rv->ai_addr, ai->ai_addr, ai->ai_addrlen);
     freeaddrinfo(ai);
-    
-    return rv;    
+
+    return rv;
 #else /* !USE_GETADDRINFO */
     struct in_addr a;
-    
+
     if (type == ne_iaddr_ipv6) {
         return NULL;
     }
@@ -1129,7 +1186,7 @@ ne_inet_addr *ne_iaddr_parse(const char *addr, ne_iaddr_type type)
         return NULL;
     }
 #endif
-    
+
     return ne_iaddr_make(ne_iaddr_ipv4, (unsigned char *)&a.s_addr);
 #endif /* !USE_GETADDRINFO */
 }
@@ -1141,7 +1198,7 @@ int ne_iaddr_reverse(const ne_inet_addr *ia, char *buf, size_t bufsiz)
                        NULL, 0, 0);
 #else
     struct hostent *hp;
-    
+
     hp = gethostbyaddr(ia, sizeof *ia, AF_INET);
     if (hp && hp->h_name) {
         ne_strnzcpy(buf, hp->h_name, bufsiz);
@@ -1200,7 +1257,7 @@ static int timed_connect(ne_socket *sock, int fd,
             set_strerror(sock, errno);
             return NE_SOCK_ERROR;
         }
-        
+
         ret = raw_connect(fd, sa, salen);
         if (ret == -1) {
             errnum = ne_errno;
@@ -1208,7 +1265,7 @@ static int timed_connect(ne_socket *sock, int fd,
                 ret = raw_poll(fd, 1, sock->cotimeout);
                 if (ret > 0) { /* poll got data */
                     socklen_t len = sizeof(errnum);
-                    
+
                     /* Check whether there is a pending error for the
                      * socket.  Per Stevens UNPv1§15.4, Solaris will
                      * return a pending error via errno by failing the
@@ -1217,7 +1274,7 @@ static int timed_connect(ne_socket *sock, int fd,
                     errnum = 0;
                     if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &errnum, &len))
                         errnum = errno;
-                    
+
                     if (errnum == 0) {
                         ret = 0;
                     } else {
@@ -1231,22 +1288,22 @@ static int timed_connect(ne_socket *sock, int fd,
                     set_strerror(sock, errno);
                     ret = NE_SOCK_ERROR;
                 }
-            } else /* non-EINPROGRESS error from connect() */ { 
+            } else /* non-EINPROGRESS error from connect() */ {
                 set_strerror(sock, errnum);
                 ret = NE_SOCK_ERROR;
             }
         }
-        
+
         /* Reset to old flags: */
         if (fcntl(fd, F_SETFL, flags) == -1) {
             set_strerror(sock, errno);
             ret = NE_SOCK_ERROR;
-        }       
-    } else 
+        }
+    } else
 #endif /* USE_NONBLOCKING_CONNECT */
     {
         ret = raw_connect(fd, sa, salen);
-        
+
         if (ret < 0) {
             set_strerror(sock, errno);
             ret = NE_SOCK_ERROR;
@@ -1316,12 +1373,12 @@ void ne_sock_prebind(ne_socket *sock, const ne_inet_addr *addr,
                      unsigned int port)
 {
     sock->lport = port;
-    sock->laddr = addr ? addr : &dummy_laddr;    
+    sock->laddr = addr ? addr : &dummy_laddr;
 }
 
 /* Bind socket 'fd' to address/port 'addr' and 'port', for subsequent
  * connect() to address of family 'peer_family'. */
-static int do_bind(int fd, int peer_family, 
+static int do_bind(int fd, int peer_family,
                    const ne_inet_addr *addr, unsigned int port)
 {
 #if defined(HAVE_SETSOCKOPT) && defined(SO_REUSEADDR) && defined(SOL_SOCKET)
@@ -1331,8 +1388,8 @@ static int do_bind(int fd, int peer_family,
         (void) setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof flag);
         /* An error here is not fatal, so ignore it. */
     }
-#endif        
-    
+#endif
+
 
 #if defined(USE_GETADDRINFO) && defined(AF_INET6)
     /* Use a sockaddr_in6 if an AF_INET6 local address is specifed, or
@@ -1340,7 +1397,7 @@ static int do_bind(int fd, int peer_family,
     if ((addr != &dummy_laddr && addr->ai_family == AF_INET6)
         || (addr == &dummy_laddr && peer_family == AF_INET6)) {
         struct sockaddr_in6 in6;
-        
+
         if (addr == &dummy_laddr)
             memset(&in6, 0, sizeof in6);
         else
@@ -1412,7 +1469,7 @@ int ne_sock_connect(ne_socket *sock,
         set_strerror(sock, ne_errno);
 	return -1;
     }
-    
+
 #if !defined(NE_USE_POLL) && !defined(WIN32)
     if (fd > FD_SETSIZE) {
         ne_close(fd);
@@ -1420,7 +1477,7 @@ int ne_sock_connect(ne_socket *sock,
         return NE_SOCK_ERROR;
     }
 #endif
-   
+
 #if defined(HAVE_FCNTL) && defined(F_GETFD) && defined(F_SETFD) \
   && defined(FD_CLOEXEC)
     /* Set the FD_CLOEXEC bit for the new fd, if the socket was not
@@ -1431,7 +1488,7 @@ int ne_sock_connect(ne_socket *sock,
     }
 #endif
 
-    if (sock->laddr && (sock->laddr == &dummy_laddr || 
+    if (sock->laddr && (sock->laddr == &dummy_laddr ||
                         ia_family(sock->laddr) == ia_family(addr))) {
         ret = do_bind(fd, ia_family(addr), sock->laddr, sock->lport);
         if (ret < 0) {
@@ -1481,7 +1538,7 @@ ne_inet_addr *ne_sock_peer(ne_socket *sock, unsigned int *port)
         set_error(sock, _("Socket family not supported"));
         return NULL;
     }
-#endif                  
+#endif
 
     ia = ne_calloc(sizeof *ia);
 #ifdef USE_GETADDRINFO
@@ -1491,10 +1548,10 @@ ne_inet_addr *ne_sock_peer(ne_socket *sock, unsigned int *port)
     ia->ai_family = saun.sa.sa_family;
 #else
     memcpy(ia, &saun.sin.sin_addr.s_addr, sizeof *ia);
-#endif    
+#endif
 
 #if defined(USE_GETADDRINFO) && defined(AF_INET6)
-    *port = ntohs(saun.sa.sa_family == AF_INET ? 
+    *port = ntohs(saun.sa.sa_family == AF_INET ?
                   saun.sin.sin_port : saun.sin6.sin6_port);
 #else
     *port = ntohs(saun.sin.sin_port);
@@ -1535,7 +1592,7 @@ ne_inet_addr *ne_iaddr_make(ne_iaddr_type type, const unsigned char *raw)
 #endif
 #else /* !USE_GETADDRINFO */
     memcpy(&ia->s_addr, raw, sizeof ia->s_addr);
-#endif    
+#endif
     return ia;
 }
 
@@ -1554,18 +1611,18 @@ int ne_iaddr_cmp(const ne_inet_addr *i1, const ne_inet_addr *i2)
     if (i1->ai_family != i2->ai_family)
 	return i2->ai_family - i1->ai_family;
     if (i1->ai_family == AF_INET) {
-	struct sockaddr_in *in1 = SACAST(in, i1->ai_addr), 
+	struct sockaddr_in *in1 = SACAST(in, i1->ai_addr),
 	    *in2 = SACAST(in, i2->ai_addr);
-	return memcmp(&in1->sin_addr.s_addr, &in2->sin_addr.s_addr, 
+	return memcmp(&in1->sin_addr.s_addr, &in2->sin_addr.s_addr,
 		      sizeof in1->sin_addr.s_addr);
-    } 
+    }
 #ifdef AF_INET6
     else if (i1->ai_family == AF_INET6) {
-	struct sockaddr_in6 *in1 = SACAST(in6, i1->ai_addr), 
+	struct sockaddr_in6 *in1 = SACAST(in6, i1->ai_addr),
 	    *in2 = SACAST(in6, i2->ai_addr);
 	return memcmp(in1->sin6_addr.s6_addr, in2->sin6_addr.s6_addr,
 		      sizeof in1->sin6_addr.s6_addr);
-    } 
+    }
 #endif /* AF_INET6 */
     else
 	return -1;
@@ -1582,7 +1639,7 @@ void ne_iaddr_free(ne_inet_addr *addr)
     ne_free(addr);
 }
 
-int ne_sock_accept(ne_socket *sock, int listener) 
+int ne_sock_accept(ne_socket *sock, int listener)
 {
     int fd = accept(listener, NULL, NULL);
 
@@ -1626,7 +1683,7 @@ static int store_sess(void *userdata, gnutls_datum key, gnutls_datum data)
 {
     ne_ssl_context *ctx = userdata;
 
-    if (ctx->cache.server.key.data) { 
+    if (ctx->cache.server.key.data) {
         gnutls_free(ctx->cache.server.key.data);
         gnutls_free(ctx->cache.server.data.data);
     }
@@ -1672,7 +1729,7 @@ int ne_sock_accept_ssl(ne_socket *sock, ne_ssl_context *ctx)
 
 #if defined(HAVE_OPENSSL)
     ssl = SSL_new(ctx->ctx);
-    
+
     SSL_set_fd(ssl, sock->fd);
 
     sock->ssl = ssl;
@@ -1691,8 +1748,8 @@ int ne_sock_accept_ssl(ne_socket *sock, ne_ssl_context *ctx)
 
     /* Set up dummy session cache. */
     gnutls_db_set_store_function(ssl, store_sess);
-    gnutls_db_set_retrieve_function(ssl, retrieve_sess);    
-    gnutls_db_set_remove_function(ssl, remove_sess);    
+    gnutls_db_set_retrieve_function(ssl, retrieve_sess);
+    gnutls_db_set_remove_function(ssl, remove_sess);
     gnutls_db_set_ptr(ssl, ctx);
 
     if (ctx->verify)
@@ -1715,7 +1772,7 @@ int ne_sock_accept_ssl(ne_socket *sock, ne_ssl_context *ctx)
 
 int ne_sock_connect_ssl(ne_socket *sock, ne_ssl_context *ctx, void *userdata)
 {
-    int ret;
+    int ret = -1;
 
 #if defined(HAVE_OPENSSL)
     SSL *ssl;
@@ -1738,7 +1795,26 @@ int ne_sock_connect_ssl(ne_socket *sock, ne_ssl_context *ctx, void *userdata)
 	set_error(sock, _("Could not create SSL structure"));
 	return NE_SOCK_ERROR;
     }
-    
+
+    // check sock->fd, if it's blocking set to non-blocking just for SSL_conect
+    long old_flags, new_flags = 0;
+    time_t timeout;
+
+    if (sock->cotimeout)
+        timeout = time(0) + sock->cotimeout;
+
+    // keep the old flags so we can reset them later
+    new_flags = old_flags = fcntl(sock->fd, F_GETFL);
+
+    if(!(new_flags & O_NONBLOCK)){
+        new_flags |= O_NONBLOCK;
+
+        if(fcntl(sock->fd, F_SETFL, new_flags) == -1){
+            set_strerror(sock, errno);
+            return NE_SOCK_ERROR;
+        }
+    }
+
     SSL_set_app_data(ssl, userdata);
     SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY);
     SSL_set_fd(ssl, sock->fd);
@@ -1754,17 +1830,107 @@ int ne_sock_connect_ssl(ne_socket *sock, ne_ssl_context *ctx, void *userdata)
         }
     }
 #endif
-    
+
+
+
     if (ctx->sess)
 	SSL_set_session(ssl, ctx->sess);
-
+/*
     ret = SSL_connect(ssl);
     if (ret != 1) {
-	error_ossl(sock, ret);
-	SSL_free(ssl);
-	sock->ssl = NULL;
-	return NE_SOCK_ERROR;
+        error_ossl(sock, ret);
+        SSL_free(ssl);
+        sock->ssl = NULL;
+        return NE_SOCK_ERROR;
     }
+*/
+
+    // non-blocking SSL handshake
+#ifdef NE_USE_POLL
+    int polltimeout = 1000;
+#else
+    struct timeval tv;
+#endif
+
+    while((ret = SSL_connect(ssl)) != 1){
+#ifdef NE_USE_POLL
+        struct pollfd pfd;
+        pfd.fd = sock->fd;
+#else
+        fd_set fds;
+        FD_ZERO(&fds);
+        FD_SET(sock->fd, &fds);
+
+        tv.tv_sec = 1;
+        tv.tv_usec = 0;
+#endif
+
+        switch(SSL_get_error(ssl, ret)){
+#ifdef NE_USE_POLL
+            case SSL_ERROR_WANT_WRITE:
+                pfd.events = POLLOUT;
+                poll(&pfd, 1, polltimeout);
+                break;
+
+            case SSL_ERROR_WANT_READ:
+                pfd.events = POLLIN;
+                poll(&pfd, 1, polltimeout);
+                break;
+
+            case SSL_ERROR_WANT_X509_LOOKUP:
+                pfd.events = POLLIN | POLLOUT;
+                poll(&pfd, 1, polltimeout);
+                break;
+#else
+            case SSL_ERROR_WANT_WRITE:
+                select(sock->fd +1, NULL, &fds, NULL, &tv);
+                break;
+
+            case SSL_ERROR_WANT_READ:
+                select(sock->fd +1, &fds, NULL, NULL, &tv);
+                break;
+
+            case SSL_ERROR_WANT_X509_LOOKUP:
+                select(sock->fd +1, &fds, &fds, NULL, &tv);
+                break;
+#endif
+
+            case SSL_ERROR_NONE:
+            case SSL_ERROR_ZERO_RETURN:
+            case SSL_ERROR_WANT_CONNECT:
+            case SSL_ERROR_WANT_ACCEPT:
+            case SSL_ERROR_SYSCALL:
+                break;
+            case SSL_ERROR_SSL:{
+                int ssl_err_code = ERR_get_error();
+                const char* ssl_err_message = ERR_reason_error_string(ssl_err_code);
+                ERR_clear_error();
+                set_error(sock, _(ssl_err_message));
+                return NE_SOCK_ERROR;
+            }
+
+            default:
+                set_error(sock, _("Unknown error during SSL handshake"));
+                return NE_SOCK_ERROR;
+        }
+
+        if(time(0) > timeout){
+            error_ossl(sock, ret);
+            SSL_free(ssl);
+            sock->ssl = NULL;
+            set_error(sock, _("Connection timed out during SSL handshake"));
+            return NE_SOCK_ERROR;
+        }
+    }
+
+    // reset flags if needed
+    if(new_flags != old_flags){
+        if(fcntl(sock->fd, F_SETFL, old_flags) == -1){
+            set_strerror(sock, errno);
+            return NE_SOCK_ERROR;
+        }
+    }
+
 #elif defined(HAVE_GNUTLS)
     /* DH and RSA params are set in ne_ssl_context_create */
     gnutls_init(&sock->ssl, GNUTLS_CLIENT);
@@ -1774,24 +1940,24 @@ int ne_sock_connect_ssl(ne_socket *sock, ne_ssl_context *ctx, void *userdata)
 
 #ifdef HAVE_GNUTLS_SIGN_CALLBACK_SET
     if (ctx->sign_func)
-        gnutls_sign_callback_set(sock->ssl, ctx->sign_func, ctx->sign_data);    
+        gnutls_sign_callback_set(sock->ssl, ctx->sign_func, ctx->sign_data);
 #endif
 
     if (ctx->hostname) {
         gnutls_server_name_set(sock->ssl, GNUTLS_NAME_DNS, ctx->hostname,
                                strlen(ctx->hostname));
-    }                               
+    }
 
     gnutls_transport_set_ptr(sock->ssl, (gnutls_transport_ptr)(long)sock->fd);
 
     if (ctx->cache.client.data) {
 #if defined(HAVE_GNUTLS_SESSION_GET_DATA2)
-        gnutls_session_set_data(sock->ssl, 
-                                ctx->cache.client.data, 
+        gnutls_session_set_data(sock->ssl,
+                                ctx->cache.client.data,
                                 ctx->cache.client.size);
 #else
-        gnutls_session_set_data(sock->ssl, 
-                                ctx->cache.client.data, 
+        gnutls_session_set_data(sock->ssl,
+                                ctx->cache.client.data,
                                 ctx->cache.client.len);
 #endif
     }
@@ -1812,10 +1978,10 @@ int ne_sock_connect_ssl(ne_socket *sock, ne_ssl_context *ctx, void *userdata)
         gnutls_session_get_data2(sock->ssl, &ctx->cache.client);
 #else
         ctx->cache.client.len = 0;
-        if (gnutls_session_get_data(sock->ssl, NULL, 
+        if (gnutls_session_get_data(sock->ssl, NULL,
                                     &ctx->cache.client.len) == 0) {
             ctx->cache.client.data = ne_malloc(ctx->cache.client.len);
-            gnutls_session_get_data(sock->ssl, ctx->cache.client.data, 
+            gnutls_session_get_data(sock->ssl, ctx->cache.client.data,
                                     &ctx->cache.client.len);
         }
 #endif
@@ -1850,16 +2016,19 @@ int ne_sock_sessid(ne_socket *sock, unsigned char *buf, size_t *buflen)
     sess = SSL_get0_session(sock->ssl);
 
     if (!buf) {
-        *buflen = sess->session_id_length;
+        SSL_SESSION_get_id(sess, buflen);
         return 0;
     }
 
-    if (*buflen < sess->session_id_length) {
+    unsigned int session_id_length;
+    const unsigned char* session_id = SSL_SESSION_get_id(sess, &session_id_length);
+
+    if (*buflen < session_id_length) {
         return -1;
     }
 
-    *buflen = sess->session_id_length;
-    memcpy(buf, sess->session_id, *buflen);
+    *buflen = session_id_length;
+    memcpy(buf, session_id, *buflen);
     return 0;
 #endif
 #else
@@ -1879,11 +2048,11 @@ char *ne_sock_cipher(ne_socket *sock)
         return ne_strdup(name);
 #endif
     }
-    else 
+    else
 #endif /* NE_HAVE_SSL */
     {
         return NULL;
-    }    
+    }
 }
 
 const char *ne_sock_error(const ne_socket *sock)
