@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,6 @@
  *
 */
 
-
 #ifndef DAVIXCONTEXT_HPP
 #define DAVIXCONTEXT_HPP
 
@@ -75,10 +75,10 @@ public:
     ///
     virtual ~Context();
 
-    /// clone this instance to a new context dynamically allocated,
+    /// clone this instance to a new context
     Context* clone();
 
-#ifndef DAVIX_STD_CXX03
+#ifdef __DAVIX_HAS_STD_FUNCTION
 
     /// set a new hook (callback) to intercept event in davix
     /// see davix_hooks.hpp for more details about hooks
@@ -110,6 +110,9 @@ public:
     /// get session caching status
     bool getSessionCaching() const;
 
+    /// clear both redirect and session cache
+    void clearCache();
+
 private:
     // internal context
     ContextInternal* _intern;
@@ -131,10 +134,19 @@ private:
 };
 
 
-/// version string of the current davix library
+/// runtime version string of the current davix library
 /// @return version of davix
 DAVIX_EXPORT const std::string & version();
 
+/// runtime version string of the current HTTP backend library
+/// @return version of backend library
+DAVIX_EXPORT const std::string backendRuntimeVersion();
+
+/// backend version _at the time of compilation_ which could
+/// be different than runtime one
+/// @return header version of backend library
+DAVIX_EXPORT const std::string backendHeadersVersion();
+
 }
 
 #endif // DAVIXCONTEXT_HPP
