@@ -56,7 +56,11 @@ namespace RequestProtocol{
         // Use Microsoft Azure API
         Azure,
         // Use gcloud API
-        Gcloud
+        Gcloud,
+        // Use Swift API
+        Swift,
+        // Use CS3 API
+        CS3
     };
 }
 
@@ -84,6 +88,15 @@ namespace S3ListingMode{
     };
 }
 
+namespace SwiftListingMode{
+    enum SwiftListingMode{
+        // Full hierarchical listing (depth is 1)
+        Hierarchical,
+        // Semi-hierarchical listing (depth is infinity)
+        SemiHierarchical
+    };
+}
+
 namespace CopyMode{
     enum CopyMode{
         // source push to destination
@@ -110,11 +123,31 @@ typedef std::string AwsRegion;
 ///
 typedef std::string AwsToken;
 
+///
+/// \brief string for Reva security token
+///
+typedef std::string RevaToken;
+
 ///
 /// \brief string for Azure private key
 ///
 typedef std::string AzureSecretKey;
 
+///
+/// \brief string for Openstack token
+///
+typedef std::string OSToken;
+
+///
+/// \brief string for Openstack project ID
+///
+typedef std::string OSProjectID;
+
+///
+/// \brief string for Swift Account
+///
+typedef  std::string SwiftAccount;
+
 #ifdef __DAVIX_HAS_STD_FUNCTION
 
     ///
