@@ -24,6 +24,7 @@
 
 #include <vector>
 #include <string>
+#include <map>
 
 #include "davix_request_params_types.hpp"
 #include "../auth/davixauth.hpp"
@@ -47,6 +48,12 @@ namespace gcloud {
     class Credentials;
 }
 
+namespace reva {
+    class Credentials; 
+    struct Credential;
+    typedef std::map<std::string, Credential> CredentialMap;
+}
+
 struct RequestParamsInternal;
 
 
@@ -160,6 +167,31 @@ public:
     ///
     const AwsToken & getAwsToken() const;
 
+    ///
+    /// \brief define the Reva Credentials
+    /// \param credentials credential map of type reva::Credential
+    ///
+    void setRevaCredentials(Davix::reva::Credentials & credentials);
+
+    ///
+    /// \brief get Reva Token
+    /// \return reva token of type std::string
+    ///
+    const RevaToken getRevaToken(std::string uri) const&;
+
+    ///
+    /// \brief  get Reva Credentials
+    /// \return credential map of type std::map<std::string, Credential> CredentialMap
+    /// \n      Credential consists of a RevaToken and a write_access flag
+    ///
+    const Davix::reva::Credentials  & getRevaCredentials() const;
+
+    ///
+    /// \brief get Reva Credentials
+    /// \return credentials type reva::Credential
+    ///
+    void getRevaCredentialMap(Davix::reva::CredentialMap & cmap);
+
     ///
     /// \brief set whether we're using an S3 path-based url
     /// \param alternate whether using an S3 path-based url
@@ -196,6 +228,41 @@ public:
     ///
     const Davix::gcloud::Credentials & getGcloudCredentials() const;
 
+    ///
+    /// \brief set the OS token used for Swift authentication
+    /// \param token the OS token
+    ///
+    void setOSToken(const OSToken & token);
+
+    ///
+    /// \brief get the OS token used for Swift authentication
+    /// \return the OS token
+    ///
+    const OSToken & getOSToken() const;
+
+    ///
+    /// \brief set the OS project id used for Swift authentication
+    /// \param the project id
+    ///
+    void setOSProjectID(const OSProjectID & id);
+
+    ///
+    /// \brief get the OS project id used for Swift authentication
+    /// \return the project id
+    ///
+    const OSProjectID & getOSProjectID() const;
+
+    ///
+    /// \brief set the Swift account used for Swift authentication
+    /// \param the Swift account
+    ///
+    void setSwiftAccount(const SwiftAccount & account);
+
+    ///
+    /// \brief get the Swift account used for Swift authentication
+    /// \return the Swift account
+    ///
+    const SwiftAccount & getSwiftAccount() const;
 
     /// set listing mode flag for S3 bucket
     void setS3ListingMode(const S3ListingMode::S3ListingMode s3_listing_mode);
@@ -203,6 +270,12 @@ public:
     /// get listing mode flag for S3 bucket
     S3ListingMode::S3ListingMode getS3ListingMode() const;
 
+    /// set listing mode flag for Swift
+    void setSwiftListingMode(const SwiftListingMode::SwiftListingMode swift_listing_mode);
+
+    /// get listing mode flag for Swift
+    SwiftListingMode::SwiftListingMode getSwiftListingMode() const;
+
     /// set maximum number of key entries return by S3 list object request
     void setS3MaxKey(const unsigned long s3_max_key_entries);
 
