@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -128,13 +129,13 @@ enum Code {
 
     /// Impossible to decrypt client credential for usage
     CredDecryptionError = 0x019,
-    
+
     /// Operation canceled
     Canceled = 0x020,
-    
+
     /// Delegation error
     DelegationError = 0x021,
-    
+
     /// Remote error. Used for third party copies: it means the
     /// destination failed.
     RemoteError = 0x022,
@@ -145,6 +146,18 @@ enum Code {
     /// Invalid Hook type
     InvalidHook = 0x24,
 
+    /// Connection timeout during a redirection
+    TimeoutRedirectionError = 0x25,
+
+    /// Too many redirects
+    TooManyRedirects = 0x26,
+
+    /// Insufficient storage
+    InsufficientStorage = 0x27,
+
+    /// Environment Variable Missing
+    EnvVarNotSet = 0x28,
+
     /// Undefined error
     UnknowError = 0x100
 
@@ -315,6 +328,7 @@ protected:
     DavixExceptionIntern* d_ptr;
 };
 
+
 void checkDavixError(DavixError** err);
 
 
@@ -333,6 +347,8 @@ void checkDavixError(DavixError** err);
 DAVIX_EXPORT std::string davix_scope_stat_str();
 DAVIX_EXPORT std::string davix_scope_davOps_str();
 DAVIX_EXPORT std::string davix_scope_mkdir_str();
+DAVIX_EXPORT std::string davix_scope_rm_str();
+DAVIX_EXPORT std::string davix_scope_mv_str();
 DAVIX_EXPORT std::string davix_scope_directory_listing_str();
 DAVIX_EXPORT std::string davix_scope_http_request();
 DAVIX_EXPORT std::string davix_scope_meta();
@@ -346,6 +362,8 @@ DAVIX_EXPORT std::string davix_scope_x509cred();
 //
 DAVIX_EXPORT void davix_errno_to_davix_error(int errcode, const std::string & scope, const std::string & msg, DavixError** newErr);
 
+DAVIX_EXPORT void errno_to_davix_exception(int errno_code, const std::string & scope, const std::string & msg);
+
 
 
 
