@@ -22,8 +22,8 @@
 #include <davix_internal.hpp>
 #include "httpiovec.hpp"
 #include <utils/davix_logger_internal.hpp>
-#include <string_utils/stringutils.hpp>
-#include <IntervalTree.h>
+#include <utils/stringutils.hpp>
+#include "libs/IntervalTree.h"
 
 #include <map>
 
