@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,15 +19,16 @@
  *
 */
 
-#include <config.h>
+#include <davix_internal_config.hpp>
 #include "iobuffmap.hpp"
 
+#include <core/ContentProvider.hpp>
 #include <utils/davix_types.hpp>
 #include <request/httprequest.hpp>
 #include <utils/davix_logger_internal.hpp>
-#include <http_util/http_util.hpp>
 #include <fileops/httpiovec.hpp>
 #include <fileops/davmeta.hpp>
+#include <system_utils/env_utils.hpp>
 
 
 #include <sstream>
@@ -44,21 +46,19 @@
 
 namespace Davix {
 
-
-dav_ssize_t read_segment_request(HttpRequest* req, void* buffer, dav_size_t size_read,  dav_off_t off_set, DavixError**err);
 dav_ssize_t read_truncated_segment_request(HttpRequest* req, void* buffer, dav_size_t size_read,  dav_off_t off_set, DavixError**err);
 
 
 
 
-dav_ssize_t read_segment_request(HttpRequest* req, void* buffer, dav_size_t size_read,  dav_off_t off_set, DavixError**err){
+dav_ssize_t read_segment_request(HttpRequest* req, void* buffer, dav_size_t size_read, DavixError**err){
     DavixError* tmp_err=NULL;
     dav_ssize_t ret, tmp_ret;
     char* p_buff =(char*) buffer;
     dav_size_t s_read= size_read;
     ret = tmp_ret = 0;
 
-    DAVIX_TRACE("Davix::IOMap::readSegment: want to read %lld bytes ", size_read);
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Davix::IOMap::readSegment: want to read {} bytes ", size_read);
     do{
         tmp_ret= req->readBlock(p_buff, s_read, &tmp_err);
         if(tmp_ret > 0){ // tmp_ret bytes readed
@@ -75,7 +75,7 @@ dav_ssize_t read_segment_request(HttpRequest* req, void* buffer, dav_size_t size
         DavixError::propagateError(err, tmp_err);
         return -1;
     }
-    DAVIX_TRACE("Davix::IOMap::readSegment: got %lld bytes ", ret);
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Davix::IOMap::readSegment: got {} bytes ", ret);
     return ret;
 }
 
@@ -99,7 +99,7 @@ dav_ssize_t read_truncated_segment_request(HttpRequest* req, void* buffer, dav_s
      }
 
      if(!tmp_err){
-        ret = read_segment_request(req, p_buffer, size_read,  off_set, &tmp_err);
+        ret = read_segment_request(req, p_buffer, size_read, &tmp_err);
      }
 
      if(tmp_err){
@@ -144,7 +144,8 @@ dav_ssize_t HttpIO::readFull(IOChainContext & iocontext, std::vector<char> & buf
     DavixError * tmp_err=NULL;
     dav_ssize_t ret = -1, total=0;
 
-    DAVIX_DEBUG(" -> readFull on vector");
+    DAVIX_SCOPE_TRACE(DAVIX_LOG_CHAIN, fun_readFull);
+
     GetRequest req (iocontext._context, iocontext._uri, &tmp_err);
     if(!tmp_err){
         RequestParams params(iocontext._reqparams);
@@ -158,22 +159,21 @@ dav_ssize_t HttpIO::readFull(IOChainContext & iocontext, std::vector<char> & buf
                 total += (dav_size_t) ret;
             }
             if(!tmp_err && httpcodeIsValid(req.getRequestCode()) == false){
-                httpcodeToDavixCode(req.getRequestCode(),davix_scope_io_buff(),"read error: ", &tmp_err);
+                httpcodeToDavixError(req.getRequestCode(),davix_scope_io_buff(),"read error: ", &tmp_err);
                 ret = -1;
             }
         }
     }
 
-    DAVIX_DEBUG(" <- readFull on vector");
     checkDavixError(&tmp_err);
-    return (ret>0)?total:-1;
+    return (ret>=0)?total:-1;
 }
 
 
 dav_ssize_t HttpIO::pread(IOChainContext & iocontext, void *buf, dav_size_t count, dav_off_t offset){
     DavixError * tmp_err=NULL;
     dav_ssize_t ret = -1;
-    DAVIX_DEBUG(" -> getOps operation for %s with size %ld and offset %ld", iocontext._uri.getString().c_str(), count, offset);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "pread operation for {} with size {} and offset {}", iocontext._uri, count, offset);
     if(count ==0)
         return 0;
 
@@ -185,37 +185,57 @@ dav_ssize_t HttpIO::pread(IOChainContext & iocontext, void *buf, dav_size_t coun
         if(req.beginRequest(&tmp_err) ==0){
             if(req.getRequestCode() == 416 ){ // out of file, end of file
                 ret = 0; // end of file
+                DavixError::clearError(&tmp_err);
             }else{
                 if(req.getRequestCode() == 206 ){ // partial request supported, just read !
-                    ret = read_segment_request(&req, buf, count, offset, &tmp_err);
+                    ret = read_segment_request(&req, buf, count, &tmp_err);
+
+                    // clean remaining content
+                    if(!tmp_err) {
+                        char buffer[255];
+                        while( req.readBlock(buffer, 255, NULL) > 0);
+                    }
+
                 }else if( req.getRequestCode() == 200){ // full request content -> skip useless content
                     ret = read_truncated_segment_request(&req, buf, count, offset, &tmp_err);
                 }else{
-                    httpcodeToDavixCode(req.getRequestCode(),davix_scope_http_request(),", while  readding", &tmp_err);
+                    httpcodeToDavixError(req.getRequestCode(), davix_scope_http_request(),", while  readding", &tmp_err);
                 }
             }
         }
         req.endRequest(NULL);
     }
-    DAVIX_DEBUG(" end getOps operation for %s <- ",iocontext._uri.getString().c_str());
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "end pread operation for {} ",iocontext._uri);
     checkDavixError(&tmp_err);
     return ret;
 }
 
+#define SSTR(message) static_cast<std::ostringstream&>(std::ostringstream().flush() << message).str()
 
 dav_ssize_t HttpIO::readToFd(IOChainContext & iocontext, int fd, dav_size_t read_size){
     DavixError * tmp_err=NULL;
     dav_ssize_t ret = -1;
 
-    DAVIX_DEBUG(" -> readToFd for size %ld", read_size);
+    if(iocontext.fdHandler.fd != fd) {
+        iocontext.fdHandler.fd = fd;
+        iocontext.fdHandler.bytes_written_to_fd = 0;
+    }
+
+    DAVIX_SCOPE_TRACE(DAVIX_LOG_CHAIN, fun_readToFd);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "request size {}", read_size);
     GetRequest req (iocontext._context, iocontext._uri, &tmp_err);
     if(!tmp_err){
         RequestParams params(iocontext._reqparams);
         req.setParameters(iocontext._reqparams);
+        if(iocontext.fdHandler.bytes_written_to_fd > 0) {
+            DAVIX_SLOG(DAVIX_LOG_WARNING, DAVIX_LOG_CHAIN, "{} bytes were already written to fd before transfer failed; attempting to resume from that point on", iocontext.fdHandler.bytes_written_to_fd);
+            req.addHeaderField("Range", SSTR("bytes=" << iocontext.fdHandler.bytes_written_to_fd << "-"));
+        }
+
         ret = req.beginRequest(&tmp_err);
         if(!tmp_err){
             if(httpcodeIsValid(req.getRequestCode()) == false){
-                httpcodeToDavixCode(req.getRequestCode(),davix_scope_io_buff(),"read error: ", &tmp_err);
+                httpcodeToDavixError(req.getRequestCode(),davix_scope_io_buff(),"read error: ", &tmp_err);
                 ret = -1;
             }else{
                 ret= req.readToFd(fd, read_size, &tmp_err);
@@ -223,41 +243,37 @@ dav_ssize_t HttpIO::readToFd(IOChainContext & iocontext, int fd, dav_size_t read
         }
     }
 
+    if(ret > 0) {
+        iocontext.fdHandler.bytes_written_to_fd += ret;
+    }
 
-    DAVIX_DEBUG(" <- readToFd for size %ld", read_size);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "read size {}", ret);
     checkDavixError(&tmp_err);
     return ret;
 }
 
-
-// position independant write operation,
-// similar to pwrite do not need open() before
-dav_ssize_t HttpIO::writeFromFd(IOChainContext & iocontext, int fd, dav_size_t size){
+dav_ssize_t HttpIO::writeFromProvider(IOChainContext & iocontext, ContentProvider &provider) {
     DavixError * tmp_err=NULL;
-    dav_ssize_t ret = -1;
 
-    DAVIX_DEBUG(" -> writeFromFd for size %ld", size);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "write size {}", provider.getSize());
     PutRequest req (iocontext._context,iocontext._uri, &tmp_err);
     if(!tmp_err){
         RequestParams params(iocontext._reqparams);
         req.setParameters(params);
-        req.setRequestBody(fd,0, size);
-        ret = req.executeRequest(&tmp_err);
+        req.setRequestBody(provider);
+        req.executeRequest(&tmp_err);
         if(!tmp_err && httpcodeIsValid(req.getRequestCode()) == false){
-            httpcodeToDavixCode(req.getRequestCode(), davix_scope_io_buff(),
-                                "read error: ", &tmp_err);
-            ret = -1;
+            httpcodeToDavixError(req.getRequestCode(), davix_scope_io_buff(),
+                                "write error: ", &tmp_err);
         }
     }
 
-
-    DAVIX_DEBUG(" <- writeFromFd for size %ld", size);
+    DAVIX_SLOG(DAVIX_LOG_DEBUG, DAVIX_LOG_CHAIN, "write result size {}", provider.getSize());
     checkDavixError(&tmp_err);
-    return ret;
+    return provider.getSize();
 }
 
 
-
 /////////////////////////////////////////////////////
 /////////////////////////////////////////////////////
 ///
@@ -266,8 +282,9 @@ dav_ssize_t HttpIO::writeFromFd(IOChainContext & iocontext, int fd, dav_size_t s
 struct IOBufferLocalFile{
     IOBufferLocalFile(int fd, const std::string & filepath): _fd(fd), _filepath(filepath){}
     virtual ~IOBufferLocalFile(){
-        DAVIX_TRACE("Delete tmp file %s", _filepath.c_str());
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Delete tmp file {}", _filepath);
         unlink(_filepath.c_str());
+        close(_fd);
     }
 
     int _fd;
@@ -294,11 +311,15 @@ HttpIOBuffer::~HttpIOBuffer(){
 }
 
 IOBufferLocalFile* createLocalBuffer(){
+    std::string staging = EnvUtils::getEnv("DAVIX_STAGING_AREA", "/tmp");
+    staging += "/.davix_tmp_file_XXXXXX";
+
     char buffer[1024];
-    strcpy(buffer, "/tmp/.davix_tmp_file_XXXXXXXXXXXXXXXXXXXXXXXX");
+    strncpy(buffer, staging.c_str(), 1023);
+
     int fd;
     if( (fd = mkstemp(buffer)) < 0){
-        DAVIX_TRACE("Error during temporary file creation for HTTPIO %s: %s", buffer, strerror(errno));
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Error during temporary file creation for HTTPIO {}: {}", buffer, strerror(errno));
         return NULL;
     }
     return new IOBufferLocalFile(fd, buffer);
@@ -335,12 +356,12 @@ bool HttpIOBuffer::open(IOChainContext & iocontext, int flags){
         }
     }
 
-    DAVIX_TRACE("File open %s, size: %ld", iocontext._uri.getString().c_str(), _file_size);
+    DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "File open {}, size: {}", iocontext._uri, _file_size);
     return res;
 }
 
 dav_ssize_t HttpIOBuffer::read(IOChainContext & iocontext, void *buf, dav_size_t count){
-    boost::recursive_mutex::scoped_lock l(_rwlock);
+    std::lock_guard<std::recursive_mutex> l(_rwlock);
     DavixError* tmp_err = NULL;
     dav_ssize_t ret =-1;
 
@@ -375,7 +396,7 @@ dav_ssize_t HttpIOBuffer::readInternal(IOChainContext & iocontext, void *buffer,
         _read_req->setParameters(params);
         if(_read_req->beginRequest(&tmp_err) ==0
             && (_read_req->getRequestCode() != 200)){
-                httpcodeToDavixCode(_read_req->getRequestCode(),davix_scope_http_request(),", while  readding", &tmp_err);
+                httpcodeToDavixError(_read_req->getRequestCode(),davix_scope_http_request(),", while  readding", &tmp_err);
                 delete _read_req;
                 _read_req = NULL;
 
@@ -389,7 +410,7 @@ dav_ssize_t HttpIOBuffer::readInternal(IOChainContext & iocontext, void *buffer,
     }
 
     if(_read_req != NULL){ // valid request -> proceed to read
-        ret = read_segment_request(_read_req, buffer, size_read, _read_pos, &tmp_err);
+        ret = read_segment_request(_read_req, buffer, size_read, &tmp_err);
         if(ret > 0){
             _read_pos += ret;
             if(ret < (dav_ssize_t) size_read){ // end of file
@@ -411,12 +432,15 @@ dav_ssize_t HttpIOBuffer::readInternal(IOChainContext & iocontext, void *buffer,
 
 
 void HttpIOBuffer::prefetchInfo(IOChainContext & iocontext, off_t offset, dav_size_t size_read, advise_t adv){
+    (void) iocontext;
+    (void) offset;
+    (void) size_read;
     _last_advise = adv;
 }
 
 
 void HttpIOBuffer::resetIO(IOChainContext & iocontext){
-    boost::recursive_mutex::scoped_lock l(_rwlock);
+    std::lock_guard<std::recursive_mutex> l(_rwlock);
 
     if(_read_req){
         delete _read_req;
@@ -428,20 +452,23 @@ void HttpIOBuffer::resetIO(IOChainContext & iocontext){
 
 
 void HttpIOBuffer::commitLocal(IOChainContext & iocontext){
-    boost::recursive_mutex::scoped_lock l(_rwlock);
+    std::lock_guard<std::recursive_mutex> l(_rwlock);
     if(_local.get()){
         struct stat st;
         memset(&st,0, sizeof(struct stat));
         fstat(_local->_fd, &st);
-        DAVIX_TRACE("Commit local file modifications, %d bytes", st.st_size);
-        _start->writeFromFd(iocontext, _local->_fd, st.st_size);
+        DAVIX_SLOG(DAVIX_LOG_TRACE, DAVIX_LOG_CHAIN, "Commit local file modifications, {} bytes", st.st_size);
+
+        FdContentProvider provider(_local->_fd, 0, st.st_size);
+        _start->writeFromProvider(iocontext, provider);
         _local.reset();
     }
 }
 
 
 dav_off_t HttpIOBuffer::lseek(IOChainContext & iocontext, dav_off_t offset, int flags){
-    boost::recursive_mutex::scoped_lock l(_rwlock);
+    (void) iocontext;
+    std::lock_guard<std::recursive_mutex> l(_rwlock);
     switch(flags){
         case SEEK_CUR:
             _pos += offset;
@@ -458,14 +485,19 @@ dav_off_t HttpIOBuffer::lseek(IOChainContext & iocontext, dav_off_t offset, int
     return _pos;
 }
 
+
 dav_ssize_t HttpIOBuffer::write(IOChainContext & iocontext, const void *buf, dav_size_t count){
-    boost::recursive_mutex::scoped_lock l(_rwlock);
+    (void) iocontext;
+    std::lock_guard<std::recursive_mutex> l(_rwlock);
     dav_ssize_t ret =-1;
     dav_size_t write_len = count;
 
+    if(!_opened) {
+        throw DavixException(davix_scope_io_buff(), StatusCode::SystemError, "Impossible to write, descriptor has not been opened");
+    }
 
-    if(_local.get() == NULL || !_opened){
-        throw DavixException(davix_scope_io_buff(), StatusCode::SystemError, "Impossible to write, I/O Error");
+    if(_local.get() == NULL) {
+        throw DavixException(davix_scope_io_buff(), StatusCode::SystemError, "Impossible to write, no buffer. (file was opened only for reading?)");
     }
 
     do{
@@ -486,6 +518,4 @@ dav_ssize_t HttpIOBuffer::write(IOChainContext & iocontext, const void *buf, dav
 }
 
 
-
-
 } // namespace Davix
