@@ -33,8 +33,9 @@
 #include <ne_auth.h>
 #include <neon/neonsessionfactory.hpp>
 
-#include <request/httprequest.hpp>
 #include <neon/neonsession.hpp>
+#include <backend/BoundHooks.hpp>
+#include <status/DavixStatus.hpp>
 
 #include "../backend/BackendRequest.hpp"
 
@@ -45,14 +46,14 @@ namespace Davix {
 
 class NEONSessionFactory;
 class NEONSession;
-class HttpRequest;
 class NEONSessionWrapper;
+class StandaloneRequest;
 
-class NEONRequest : public BackendRequest
+class NeonRequest : public BackendRequest
 {
 public:
-    NEONRequest(HttpRequest &h, Context& f, const Uri & uri_req);
-    virtual ~NEONRequest();
+    NeonRequest(const BoundHooks &hooks, Context& f, const Uri & uri_req);
+    virtual ~NeonRequest();
 
     //--------------------------------------------------------------------------
     // Execute request synchronously, and store result in internal buffer.
@@ -91,12 +92,32 @@ public:
     virtual size_t getAnswerHeaders(std::vector<std::pair<std::string, std::string > > & vec_headers) const;
 
 private:
+    //--------------------------------------------------------------------------
+    // Initialize and configure _neon_req
+    //--------------------------------------------------------------------------
+    void createBackendRequest();
 
-    // neon internal field
-    std::unique_ptr<NEONSessionWrapper> _neon_sess;
+    //--------------------------------------------------------------------------
+    // Check redirect cache
+    //--------------------------------------------------------------------------
+    void checkRedirectCache();
 
+    //--------------------------------------------------------------------------
+    // Prepare URI & params
+    //--------------------------------------------------------------------------
+    void prepareUriParams();
+
+    //--------------------------------------------------------------------------
+    // Initialize standalone request
+    //--------------------------------------------------------------------------
+    void initStandaloneRequest();
+
+    // neon standalone request
+    std::unique_ptr<StandaloneRequest> _standalone_req;
+
+    // bound hooks
+    BoundHooks _bound_hooks;
 
-    ne_request * _req;
     // req info
     int _number_try;
     // number of redirects so far
@@ -104,28 +125,18 @@ private:
     // read info
     dav_ssize_t _total_read_size, _last_read;
 
-    HttpRequest & _h;
-    bool req_started, req_running;
-
+    bool _headers_configured;
     int _accepted_202_retries;
 
     ////////////////////////////////////////////
     // Private Members
     int startRequest(DavixError** err);
 
-    int processRedirection(int neonCode, DavixError** err); // analyze and process redirection if needed
-
-    void resetRequest();
-
-    int instanceSession(DavixError** err);
-
-    void configureRequest();
+    int processRedirection(DavixError** err); // analyze and process redirection if needed
 
+    void configureHeaders();
     void cancelSessionReuse();
 
-    // create initial neon request object
-    int createRequest(DavixError** err);
-
     // negociate the request : authentification, redirection, name resolution
     int negotiateRequest(DavixError** err);
 
@@ -137,18 +148,7 @@ private:
 
     void freeRequest();
 
-
     void createError(int ne_status, DavixError** err);
-
-	static ssize_t neon_body_content_provider(void* userdata, char* buffer, size_t buflen);
-
-    static void neon_hook_pre_send(ne_request *req, void *userdata,
-                                           ne_buffer *header);
-
-    static void neon_hook_pre_rec(ne_request *req, void *userdata,
-                                        const ne_status *status);
-
-
     friend class NEONSessionWrapper;
 };
 
@@ -156,9 +156,6 @@ private:
 //
 // translate a neon error code to a davix one
 //
-void neon_to_davix_code(int ne_status,ne_session* sess, const std::string & scope, DavixError** err);
-
-
 void neon_simple_req_code_to_davix_code(int ne_status, ne_session* sess, const std::string & scope, DavixError** err);
 
 void configureRequestParamsProto(const Uri &uri, RequestParams &params);
