@@ -25,6 +25,7 @@
 #include <params/davixrequestparams.hpp>
 #include <libs/time_utils.h>
 #include <utils/davix_gcloud_utils.hpp>
+#include <utils/davix_cs3_utils.hpp>
 
 
 
@@ -90,6 +91,7 @@ struct RequestParamsInternal{
         _recursive_mode(false),
         _s3_listing_mode(S3ListingMode::Hierarchical),
         _s3_max_key_entries(10000),
+        _swift_listing_mode(SwiftListingMode::Hierarchical),
         _ca_path(),
         _x509_data(),
         _idlogpass(),
@@ -98,9 +100,13 @@ struct RequestParamsInternal{
         _aws_cred(),
         _aws_region(),
         _aws_token(),
+        _reva_creds(),
         _aws_alternate(false),
         _azure_key(),
         _gcloud_creds(),
+        _os_token(),
+        _os_project_id(),
+        _swift_account(),
         ops_timeout(),
         connexion_timeout(),
         agent_string(default_agent),
@@ -138,6 +144,7 @@ struct RequestParamsInternal{
         _recursive_mode(param_private._recursive_mode),
         _s3_listing_mode(param_private._s3_listing_mode),
         _s3_max_key_entries(param_private._s3_max_key_entries),
+        _swift_listing_mode(param_private._swift_listing_mode),
         _ca_path(param_private._ca_path),
         _x509_data(param_private._x509_data),
         _idlogpass(param_private._idlogpass),
@@ -146,9 +153,13 @@ struct RequestParamsInternal{
         _aws_cred(param_private._aws_cred),
         _aws_region(param_private._aws_region),
         _aws_token(param_private._aws_token),
+        _reva_creds(param_private._reva_creds),
         _aws_alternate(param_private._aws_alternate),
         _azure_key(param_private._azure_key),
         _gcloud_creds(param_private._gcloud_creds),
+        _os_token(param_private._os_token),
+        _os_project_id(param_private._os_project_id),
+        _swift_account(param_private._swift_account),
         ops_timeout(),
         connexion_timeout(),
         agent_string(param_private.agent_string),
@@ -176,6 +187,9 @@ struct RequestParamsInternal{
     // s3 bucket listing mode
     S3ListingMode::S3ListingMode _s3_listing_mode;
 
+    // swift listing mode
+    SwiftListingMode::SwiftListingMode _swift_listing_mode;
+
     // Max number of keys returned by a S3 list bucket request
     unsigned long _s3_max_key_entries;
 
@@ -191,9 +205,13 @@ struct RequestParamsInternal{
     std::pair<AwsSecretKey, AwsAccessKey> _aws_cred;
     AwsRegion _aws_region;
     AwsToken _aws_token;
+    reva::Credentials _reva_creds;
     bool _aws_alternate;
     AzureSecretKey _azure_key;
     gcloud::Credentials _gcloud_creds;
+    OSToken _os_token;
+    OSProjectID _os_project_id;
+    SwiftAccount _swift_account;
 
     // timeout management
     struct timespec ops_timeout;
@@ -379,6 +397,21 @@ const AwsToken & RequestParams::getAwsToken() const {
     return d_ptr->_aws_token;
 }
 
+void RequestParams::setRevaCredentials(reva::Credentials &credentials) {
+        d_ptr->_reva_creds = credentials;
+}
+void RequestParams::getRevaCredentialMap(reva::CredentialMap & cmap) {
+        d_ptr->_reva_creds.getCredentialMap(cmap);
+}
+
+const RevaToken RequestParams::getRevaToken(std::string uri) const& {
+        return d_ptr->_reva_creds.getToken(uri);
+}
+
+const reva::Credentials & RequestParams::getRevaCredentials() const {
+        return d_ptr->_reva_creds;
+}
+
 void RequestParams::setAwsAlternate(const bool &alternate) {
     d_ptr->_aws_alternate = alternate;
 }
@@ -403,6 +436,30 @@ const gcloud::Credentials & RequestParams::getGcloudCredentials() const {
     return d_ptr->_gcloud_creds;
 }
 
+void RequestParams::setOSToken(const OSToken &token) {
+    d_ptr->_os_token = token;
+}
+
+const OSToken & RequestParams::getOSToken() const {
+    return d_ptr->_os_token;
+}
+
+void RequestParams::setOSProjectID(const OSProjectID &id) {
+    d_ptr->_os_project_id = id;
+}
+
+const OSProjectID & RequestParams::getOSProjectID() const {
+    return d_ptr->_os_project_id;
+}
+
+void RequestParams::setSwiftAccount(const SwiftAccount &account) {
+    d_ptr->_swift_account = account;
+}
+
+const SwiftAccount & RequestParams::getSwiftAccount() const {
+    return d_ptr->_swift_account;
+}
+
 void RequestParams::setS3ListingMode(const S3ListingMode::S3ListingMode s3_listing_mode){
     d_ptr->_s3_listing_mode = s3_listing_mode;
 }
@@ -411,6 +468,14 @@ S3ListingMode::S3ListingMode RequestParams::getS3ListingMode() const{
     return d_ptr->_s3_listing_mode;
 }
 
+void RequestParams::setSwiftListingMode(const SwiftListingMode::SwiftListingMode swift_listing_mode){
+    d_ptr->_swift_listing_mode = swift_listing_mode;
+}
+
+SwiftListingMode::SwiftListingMode RequestParams::getSwiftListingMode() const{
+    return d_ptr->_swift_listing_mode;
+}
+
 void RequestParams::setS3MaxKey(const unsigned long s3_max_key_entries){
     d_ptr->_s3_max_key_entries = s3_max_key_entries;
 }
